﻿using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Dienstfahrten.Klassen;
using Microsoft.SharePoint;
using Dienstfahrten.Logging;
using Microsoft.SharePoint.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using System.Collections;

namespace Dienstfahrten.ControlTemplates.Dienstfahrten
{
    public partial class KonfigurationUserControl : UserControl
    {
        SPWeb currentWeb;
        Log log;
        private int _valiErrors;

        protected void Page_Load(object sender, EventArgs e)
        {
            currentWeb = SPContext.Current.Web;
            log = new Log();

            init();
            
            if (!Page.IsPostBack)  
            {
                loadData(currentWeb);
            }
        }

        public void loadData(SPWeb currentWeb)
        {
            //Konfiguration
            Konfiguration konf = new Konfiguration(currentWeb);
            konf = (Konfiguration) konf.getByUser(currentWeb.CurrentUser);
            if (konf != null)
            {
                InputVorname.Text = konf.Vorname;
                InputNachname.Text = konf.Nachname;
                InputPersNum.Text = konf.PersonalNummer.ToString();
                InputAbt.Text = konf.Abteilung;
                InputPersNum.Text = konf.PersonalNummer.ToString();

                if (konf.Auftragsnummer != string.Empty)
                    InputAuftrnr.Text = konf.Auftragsnummer;
                if(konf.StrasseNummer != string.Empty)
                    InputStrasseNummer.Text = konf.StrasseNummer;
                if (konf.PLZ != 0)
                    InputPLZ.Text = konf.PLZ.ToString();
                if (konf.Ort != string.Empty)
                    InputOrt.Text = konf.Ort;
                if (konf.Bank != string.Empty)
                    InputBank.Text = konf.Bank;
                if (konf.BLZ != 0)
                    InputBLZ.Text = konf.BLZ.ToString();
                if (konf.KtoNr != 0)
                    InputKtoNr.Text = konf.KtoNr.ToString();
                chbxKopie.Checked = konf.Kopie;

                //Füge User in PeoplePicker hinzu
                //Wichtig: Wenn der PeoplePicker Serverseitig ausgeblendet wird, werden seine Inhalte gelöscht
                if(konf.Empfaenger != null)
                    peopleEmpfaenger.CommaSeparatedAccounts = konf.Empfaenger.LoginName;

                //Alternativ:
                /*
                ArrayList _arrayList = new ArrayList();
                if (konf.Empfaenger != null)
                {
                        PickerEntity _pickerEntity = new PickerEntity(); // PickerEntitiy use using Microsoft.SharePoint.WebControls
                        _pickerEntity.Key = konf.Empfaenger.LoginName;
                        _pickerEntity.DisplayText = konf.Empfaenger.Name;
                        _pickerEntity.IsResolved = true;
                        _arrayList.Add(_pickerEntity);
                        peopleEmpfaenger.UpdateEntities(_arrayList);
                }
                 * */
            }
            else
            {
                string[] name = currentWeb.CurrentUser.Name.Split(',');
                if (name.Length == 3)
                {
                    InputVorname.Text = name[1].ToString().Trim();
                    InputNachname.Text = name[0].ToString().Trim();
                    InputAbt.Text = name[2].ToString().Trim();
                }
            }
        }

        /// <summary>
        /// Initialisiere Sichtbarkeint der Info/Fehler-Ausgaben
        /// </summary>
        private void init()
        {
            //Konfigurations Voreinstellungen
            lblinputVornameError.Visible = false;
            lblinputNachnameError.Visible = false;
            lblinputPersNumError.Visible = false;
            lblinputStrasseNummerError.Visible = false;
            lblinputPersNumError.Visible = false;
            lblinputAbtError.Visible = false;
            lblinputPLZError.Visible = false;
            lblinputOrtError.Visible = false;
            lblinputBankError.Visible = false;
            lblinputBLZError.Visible = false;
            lblinputKtoNrError.Visible = false;
            lblinputEmpfaengerError.Visible = false;
            lblKonfInfo.Visible = false;
            lblinputAuftrnrError.Visible = false;
        }


        /// <summary>
        /// Speichern und validieren der Benutzerdaten Konfiguration.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnVerschicken_click(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// Speichern und validieren der Benutzerdaten Konfiguration.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSaveKonf_click(object sender, EventArgs e)
        {
            #region Validierung
            string vorname = string.Empty;
            string nachname = string.Empty;
            int personalNummer = 0;
            string auftragsnummer = string.Empty;
            string abteilung = string.Empty;
            string strasseNummer = string.Empty;
            int plz = 0;
            string ort = string.Empty;
            string bank = string.Empty;
            Int64 blz = 0;
            Int64 ktoNr = 0;

            _valiErrors = 0;
            //Pflichtfelder
            vorname = helper.validateTextBoxString(lblinputVornameError, InputVorname, ref _valiErrors);
            nachname = helper.validateTextBoxString(lblinputNachnameError, InputNachname, ref _valiErrors);
            personalNummer = helper.validateTextBoxInt(lblinputPersNumError, InputPersNum, ref _valiErrors);
            abteilung = helper.validateTextBoxString(lblinputAbtError, InputAbt, ref _valiErrors);
            SPUser empfaenger = helper.validateTextBoxUser(lblinputEmpfaengerError, peopleEmpfaenger, currentWeb, ref _valiErrors);

            //Optionale Felder
            if(InputAuftrnr.Text != string.Empty)
                auftragsnummer = helper.validateTextBoxString(lblinputAuftrnrError, InputAuftrnr, ref _valiErrors);
            if(InputStrasseNummer.Text != string.Empty)
                strasseNummer = helper.validateTextBoxString(lblinputStrasseNummerError, InputStrasseNummer, ref _valiErrors);
            if (InputPLZ.Text != string.Empty)            
                plz = helper.validateTextBoxInt(lblinputPLZError, InputPLZ, ref _valiErrors);
            if (InputOrt.Text != string.Empty)
                ort = helper.validateTextBoxString(lblinputOrtError, InputOrt, ref _valiErrors);
            if (InputBank.Text != string.Empty)
                bank = helper.validateTextBoxString(lblinputBankError, InputBank, ref _valiErrors);
            if (InputBLZ.Text != string.Empty)
                blz = helper.validateTextBoxLongInt(lblinputBLZError, InputBLZ, ref _valiErrors);
            if (InputKtoNr.Text != string.Empty)
                ktoNr = helper.validateTextBoxLongInt(lblinputKtoNrError, InputKtoNr, ref _valiErrors);
            bool kopie = chbxKopie.Checked;

            if (_valiErrors != 0) return;
            #endregion

            Konfiguration konf = new Konfiguration(currentWeb);
            Konfiguration newKonf = new Konfiguration(currentWeb);
            newKonf = (Konfiguration) konf.getByUser(currentWeb.CurrentUser);
            konf.setKonfValues(vorname, nachname, strasseNummer, plz, ort, personalNummer, auftragsnummer, abteilung, bank, blz, ktoNr, empfaenger, kopie);
            if (konf.saveToList())
                lblKonfInfo.Text = helper.GetLocalizedString("d_SpeichernErfolg");
            else
                lblKonfInfo.Text = helper.GetLocalizedString("d_SpeichernFehler");

            lblKonfInfo.Visible = true;

            //Umgehe Fehler"Die Sicherheitsüberprüfung für diese Seite ist ungültig" bei erster Anmeldung.
            if(newKonf == null) {
                Response.Redirect(Request.Url.AbsolutePath);
            }

        }


    }
}
