﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Dienstfahrten.Klassen
{
    using System;
    using Microsoft.SharePoint;

    /// <summary>
    /// Stellt IDs für den Zugriff auf SharePoint-Felder bereit.
    /// </summary>
    internal static class Fields
    {
        //Felder werden nicht mehr benötigt, da das Author Feld verwendet wird

        /*
        /// <summary>
        /// Feld "Benutzer Dienstfahrtliste".
        /// </summary>
        private static readonly Guid s_benutzerDl = new Guid("{99dcd312-3f32-4610-8453-0bed853050c2}");

        /// <summary>
        /// Gibt die ID des Feldes "Benutzer" der Liste "Dienfahrtliste" zurück.
        /// </summary>
        public static Guid BenutzerDl
        {
            get { return s_benutzerDl; }
        }

        /// <summary>
        /// Feld "Benutzer Konfiguration".
        /// </summary>
        private static readonly Guid s_benutzerKonf = new Guid("{b66ae08c-21c2-4ddf-baf0-24d5e7957d1a}");

        /// <summary>
        /// Gibt die ID des Feldes "Benutzer" der Liste "Dienfahrtpers" zurück.
        /// </summary>
        public static Guid BenutzerKonf
        {
            get { return s_benutzerKonf; }
        }
        */

        /// <summary>
        /// Gibt den Wert eines Feldes eines <see cref="Microsoft.SharePoint.SPListItem"/> zurück.
        /// </summary>
        /// <typeparam name="T">Der Rückgabetyp des zurückzugebenden Wertes.</typeparam>
        /// <param name="item">Das <see cref="Microsoft.SharePoint.SPListItem"/>, dessen Feldwert
        /// zurückgegeben werden soll.</param>
        /// <param name="fieldId">Die ID des Feldes, dessen Wert zurückgegeben werden soll.</param>
        /// <param name="defaultValue">Der Standardwert, der zurückgegeben werden soll, wenn das
        /// Feld den Wert <c>null</c> enthält.</param>
        /// <returns>Den Wert des Feldes vom Typ <typeparamref name="T"/>, oder <c>null</c>.</returns>
        /// <exception cref="System.IndexOutOfRangeException">Ein Feld mit der ID <paramref name="fieldId"/>
        /// existiert nicht.</exception>
        public static T GetFieldValue<T>(SPListItem item, Guid fieldId, T defaultValue)
        {
            if (item == null || !item.Fields.Contains(fieldId))
            {
                throw new IndexOutOfRangeException(
                    string.Concat("A field with ID \"", fieldId.ToString("D"), "\" do not exist."));
            }

            return
                (item[fieldId] != null
                ? (T)Convert.ChangeType(item[fieldId], typeof(T))
                : defaultValue);
        }

    }
}
