﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SharePoint;
using Dienstfahrten.Logging;

namespace Dienstfahrten.Klassen
{
    class Konfiguration : IListenEintrag
    {
        public int ID {get; private set; }
        public string Vorname { get; private set; }
        public string Nachname { get; private set; }
        public string StrasseNummer { get; private set; }
        public int PLZ { get; private set; }
        public string Ort { get; private set; }
        public int PersonalNummer { get; private set; }
        public string Auftragsnummer { get; private set; }
        public string Abteilung { get; private set; }
        public string Bank { get; private set; }
        public Int64 BLZ { get; private set; }
        public Int64 KtoNr { get; private set; }
        public SPUser Empfaenger { get; private set; }
        public bool Kopie { get; private set; }
        public SPUser Benutzer { get; private set; }


        //Statischer Name der Liste
        public string ListName { get; private set; }
        public SPWeb currentWeb { get; private set; }
        public Log log { get; private set; }
        public SPList currentList { get; private set; }


        //Konstruktor
        public Konfiguration(SPWeb web) {
            this.ListName = "Dienstfahrten Einstellungen";                            
            this.currentWeb = web;
            this.log = new Log();
            this.currentList = currentWeb.Lists[ListName];

            setKonfValues(string.Empty, string.Empty, string.Empty, 0, string.Empty, 0, string.Empty, string.Empty, string.Empty, 0, 0, null, false);
        }



        public void setKonfValues(string vorname, string nachname, string strnr, int plz, string ort, int persnr,string auftrnr, string abt,
            string bank, Int64 blz, Int64 ktonr, SPUser empf, bool kopie)
        {
            setValues(vorname, nachname, strnr, plz, ort, persnr, auftrnr, abt, bank, blz, ktonr, empf, kopie);
        }

        public void setValues(params Object[] values){

            try
            {
                if (validateData(values) == false) throw new Exception("Validation Error");

                this.Vorname = values[0].ToString();
                this.Nachname = values[1].ToString();
                this.StrasseNummer = values[2].ToString();
                this.PLZ = (int)values[3];
                this.Ort = values[4].ToString();
                this.PersonalNummer = (int)values[5];
                this.Auftragsnummer = values[6].ToString();
                this.Abteilung = values[7].ToString();
                this.Bank = values[8].ToString();
                this.BLZ = (Int64)values[9];
                this.KtoNr = (Int64)values[10];
                this.Empfaenger = (SPUser)values[11];
                this.Kopie = (bool)values[12];
            }
            catch (Exception ex)
            {
                log.Error(ex);
                throw;
            }

        }

        /// <summary>
        /// Speichere Objekt in SharePointListe
        /// </summary>
        /// <returns>Speicherung erfolgreich?</returns>
        public bool saveToList()
        {
            bool result = false;

            try
            {
                if (validateData(Vorname, Nachname, StrasseNummer, PLZ, Ort, PersonalNummer, Auftragsnummer, Abteilung, Bank, BLZ, KtoNr, Empfaenger, Kopie) == false) 
                    throw new Exception("Validation Error");

                SPListItem currentLI = null;
                SPListItemCollection ItemColl = this.getKonfFromSPList(currentWeb.CurrentUser);

                //Wenn ein Fehler aufgetreten ist oder bereits mehr als ein Eintrag vorhanden ist, dann breche ab.
                if (ItemColl == null || ItemColl.Count > 1) return false;

                //Wenn schon ein Eintrag vorhanden ist, hole ihn
                if (ItemColl.Count == 1) currentLI = currentList.GetItemById(ItemColl[0].ID);

                if (currentLI == null)
                {
                    currentLI = currentList.Items.Add();
                    //Hole die neue ID
                    currentLI.SystemUpdate();
                }

                currentLI["Title"] = currentLI.ID;
                currentLI["Vorname"] = this.Vorname;
                currentLI["Nachname"] = this.Nachname;
                currentLI["StrasseNummer"] = this.StrasseNummer;
                currentLI["PLZ"] = this.PLZ;
                currentLI["Ort"] = this.Ort;
                currentLI["PersonalNummer"] = this.PersonalNummer;
                currentLI["Auftragsnummer"] = this.Auftragsnummer;
                currentLI["Abteilung"] = this.Abteilung;
                currentLI["Bank"] = this.Bank;
                currentLI["BLZ"] = this.BLZ;
                currentLI["KtoNr"] = this.KtoNr;
                currentLI["Empfaenger"] = this.Empfaenger;
                currentLI["Kopie"] = this.Kopie;

                //AllowUnsafeUpdates ist notwendig wenn ein Item nicht neu ist sondern die Werte aktualisiert werden.
                currentLI.Update();
                result = true;
            }
            catch (Exception ex)
            {
                result = false;
                //SPUtils.log(ex);
                log.Error(ex);
                throw;
            }

            return result;
        }

        /// <summary>
        /// Holt Eintrag aus der Liste
        /// </summary>
        /// <param name="id">ID des Eintrags</param>
        /// <returns>Eintrag in der Liste</returns>
        public Object getById(int id)
        {
            try
            {
                SPListItem currentLI = currentList.GetItemById(id); //Falls nicht vorhanden wird eine ArgumentException geworfen
                
                //Holt die Werte aus der SharePoint-Liste. Falls ein Wert leer ist wird NULL gesetzt oder ein leeres Objekt.

                this.ID = currentLI.ID;
                this.Vorname = (currentLI["Vorname"] != null) ? currentLI["Vorname"].ToString() : null;
                this.Nachname = (currentLI["Nachname"] != null) ? currentLI["Nachname"].ToString() : null;
                this.StrasseNummer = (currentLI["StrasseNummer"] != null) ? currentLI["StrasseNummer"].ToString() : null;
                this.PLZ = (currentLI["PLZ"] != null) ? Convert.ToInt32(currentLI["PLZ"].ToString()) : new Int32();
                this.Ort = (currentLI["Ort"] != null) ? currentLI["Ort"].ToString() : null;
                this.PersonalNummer = (currentLI["PersonalNummer"] != null) ? Convert.ToInt32(currentLI["PersonalNummer"].ToString()) : new Int32();
                this.Auftragsnummer = (currentLI["Auftragsnummer"] != null) ? currentLI["Auftragsnummer"].ToString() : null;
                this.Abteilung = (currentLI["Abteilung"] != null) ? currentLI["Abteilung"].ToString() : null;
                this.Bank = (currentLI["Bank"] != null) ? currentLI["Bank"].ToString() : null;
                this.BLZ = (currentLI["BLZ"] != null) ? Convert.ToInt64(currentLI["BLZ"].ToString()) : new Int64();
                this.KtoNr = (currentLI["KtoNr"] != null) ? Convert.ToInt64(currentLI["KtoNr"].ToString()) : new Int64();
                this.Empfaenger = helper.getSPUserFromField("Empfaenger", currentLI);
                this.Kopie = (currentLI["Kopie"] != null) ? bool.Parse(currentLI["Kopie"] + string.Empty) : new Boolean();
                this.Benutzer = helper.getSPUserFromFieldId(SPBuiltInFieldId.Author, currentLI);
            }
            catch (ArgumentException aex)
            {
                //Wenn kein ListItem gefunden wurde
                log.Error(aex);
                return null;
            }
            catch (Exception ex)
            {
                log.Error(ex);
                throw;
            }
            
            return this;
        }

        /// <summary>
        /// Holt Eintrag des übergebenen Benutzers aus der Liste
        /// </summary>
        /// <param name="id">User</param>
        /// <returns>Eintrag in der Liste</returns>
        public Object getByUser(SPUser user)
        {
            SPListItemCollection ItemColl = this.getKonfFromSPList(user);

            List<Object> ResultList = new List<Object>();

            //Erstelle Liste mit Objekten
            if (ItemColl != null && ItemColl.Count == 1)
                return this.getById(ItemColl[0].ID);
            else return null;
            
        }

        private SPListItemCollection getKonfFromSPList(SPUser user)
        {
            //Erstelle Abfrage
            SPQuery query = new SPQuery();
            query.Query = string.Format(@"
                                <Where>
                                    <Eq>
                                        <FieldRef Name='Author' /><Value Type='User'>{0}</Value>
                                    </Eq>
                                </Where>", user.Name);

            SPListItemCollection ItemColl = currentList.GetItems(query);
            return ItemColl;
        }

        public bool validateData(params Object[] values)
        {
            bool result = false;

            try
            {
                int error = 0;
                if (values.Length > 13) throw new IndexOutOfRangeException("Allowed size is 13");
                string testString = "a";

                if (values[0] == null || !Object.ReferenceEquals(values[0].GetType(), testString.GetType())) error++;
                if (values[1] == null || !Object.ReferenceEquals(values[1].GetType(), testString.GetType())) error++;
                if (values[2] == null || !Object.ReferenceEquals(values[2].GetType(), testString.GetType())) error++;
                if (values[3] == null || !Object.ReferenceEquals(values[3].GetType(), new Int32().GetType())) error++;
                if (values[4] == null || !Object.ReferenceEquals(values[4].GetType(), testString.GetType())) error++;
                if (values[5] == null || !Object.ReferenceEquals(values[5].GetType(), new Int32().GetType())) error++;
                if (values[6] == null || !Object.ReferenceEquals(values[6].GetType(), testString.GetType())) error++;
                if (values[7] == null || !Object.ReferenceEquals(values[7].GetType(), testString.GetType())) error++;
                if (values[8] == null || !Object.ReferenceEquals(values[8].GetType(), testString.GetType())) error++;
                if (values[9] == null || !Object.ReferenceEquals(values[9].GetType(), new Int64().GetType())) error++;
                if (values[10] == null || !Object.ReferenceEquals(values[10].GetType(), new Int64().GetType())) error++;
                //if (values[10] == null || !Object.ReferenceEquals(values[10].GetType(), currentWeb.CurrentUser.GetType())) error++;
                if (values[12] == null || !Object.ReferenceEquals(values[12].GetType(), new Boolean().GetType())) error++;

                result = (error == 0);
            }
            catch (Exception ex)
            {
                log.Error(ex);
                result = false;
            }


            return result;
        }

    }
}
